# **Outliers Handling:**  `FixOutliers` Class

## `FixOutliers`: Used To Treat Outliers <br>
### Datapoints Containing Outliers Either Can Be Imputed Or Removed

<br>

### **Approaches To Detect Outliers:**
1. **Z-Score Method:** $(x - \bar{x})/\sigma$
2. **Inter-Quartile Range:** $x>Q_3, Q_1>x$

<br>

### `FixOutliers` Has A Function Called `fit_transform` 

<br>

**Parameters of `FixOutliers`**:
1. __approach__ (str) <br>
Default Value: 'interquartile_range' <br>
Possible Values: 'interquartile_range', 'z_score' and 'all'
    - 'interquartile_range' : Outliers Will Be Detected On The Basis Of Inter-Quartile Range
    - 'z_score' : Outliers Will Be Detected On The Basis Of Z-Score
    - 'all' : Outliers Will Be Detected On The Basis Of Both, Inter-Quartile Range And Z-Score

2. __treatment__ (str) <br>
Default Value: 'impute' <br>
Possible Values: 'impute' and 'remove'
    - 'impute' : Outliers Will Be Imputed 
    - 'remove' : Datapoints With Outliers Will Get Removed

3. __imputation__ (str) <br>
Default Value: 'mean' <br>
Possible Values: 'mean', 'median', 'mode' and 'std' 
    - 'mean' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `mean` Value Of The Column Where Outliers Exist.
    - 'median' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `median` Value Of The Column Where Outliers Exist.
    - 'mode' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `mode` Value Of The Column Where Outliers Exist.
    - 'std' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `Standard Deviation` Value Of The Column Where Outliers Exist.

4. __consider_outliers__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False`
    - True: Machine Will Consider Outliers While Calculating Imputation Value  
    - False: Machine Will Not Consider Outliers While Calculating Imputation Value

5. __threshold__ (float) <br>
Default Value: 3 <br>
    - User Specifies Threshold For Z-Score Computation
    
6. __index_present__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False` 
    - `False` : User Specifies That Dataset Passed By Him/Her Doesn't Have First Column As An Index Column But An Important Column.
    - `True` :  User Specifies That Dataset Passed By Him/Her Has The First Column As An Index Column.

7. __target_split__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False`
    - `False` : Data Will Be Returned As X (Data) And y (Target)
    - `True` : Data Will Be Returned As Whole One Dataset

<br>

`FixOutliers` Has A Function Called `fit_transform` :
 
**Parameters of `FixOutliers.fit_transform`**:
- Dataset: This Dataset Will Undergo Outlier Treatment, Supported Data Structures (List, Tuple, Dictionary, Numpy.Ndarray, Pandas.DataFrame)
- Col_Names: Only These Columns Will Undergo Outlier Treatment <br>
    - Default Value: ['every']
    - User Can Specify Multiple Columns As A 1-D Tuple/List
    - User Can Specify String Or An Integer If He/She Wishes To Pass Only One Column Name
    - If User Passes ['every'], Then Every Column Will Undergo Outlier Treatment
    - Col_Names As `str` Only Applicable If Dataset Is `dict` or `pandas.DataFrame`

**Returned Output by `FixOutliers.fit_transform`**: 
- `numpy.ndarray` Where Outliers Are Handled

<br>

**Examples**:

```python
>>> from analytixhero import FixOutliers
>>> Data = {
    'Index': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    'Column1': [10, 20, 15, 25, 12, 30, 18, 35, 14, 22],
    'Column2': [5, 7, 8, 6, 20, 9, 10, 18, 17, 15],
    'Column3': [1, 2, 3, 4, 5, 20, 7, 8, 9, 10],
    'Column4': [100, 150, 125, 135, 200, 145, 155, 160, 180, 170]
    }
>>> # Dataset Can Be Passed As A List, Tuple, Dictionary, NumPy.NdArray Or Pandas.DataFrame
>>> fixOut = FixOutliers(approach='all', treatment='remove', threshold=1.2, index_present=True)
>>> Data_new = fixOut.fit_transform(Data)
>>> print(Data_new)
[[ 20   7   2 150]
 [ 15   8   3 125]
 [ 25   6   4 135]
 [ 18  10   7 155]]
```

<br>

```python
>>> from analytixhero import FixOutliers
>>> Data = {
    'Column1': [10, 20, 15, 25, 12, 30, 18, 35, 14, 22],
    'Column2': [5, 7, 8, 6, 20, 9, 10, 18, 17, 15],
    'Column3': [1, 2, 3, 4, 5, 20, 7, 8, 9, 10],
    'Column4': [100, 150, 125, 135, 200, 145, 155, 160, 180, 170]
    }
>>> # Dataset Can Be Passed As A List, Tuple, Dictionary, NumPy.NdArray Or Pandas.DataFrame
>>> fixOut = FixOutliers(approach='z_score', treatment='impute', imputation='median', threshold=0.4)
>>> Data_new = fixOut.fit_transform(Data)
>>> print(Data_new)
[[ 20.   10.    7.5 152.5]
 [ 20.   10.    7.5 150. ]
 [ 20.   10.    7.5 152.5]
 [ 20.   10.    7.5 152.5]
 [ 20.   10.    5.  152.5]
 [ 20.   10.    7.5 145. ]
 [ 18.   10.    7.  155. ]
 [ 20.   10.    8.  160. ]
 [ 20.   10.    9.  152.5]
 [ 22.   10.    7.5 152.5]]
```

<br>

```python
>>> from analytixhero import FixOutliers, combine_datasets
>>> # Check combine_datasets Documentation For Clear Understanding Of The Function
>>> from sklearn.datasets import fetch_california_housing
>>> # California Housing Dataset Is An Inbuilt Dataset In Scikit-Learn
>>> Cal = fetch_california_housing()
>>> X = Cal.data
>>> y = Cal.target
>>> Data = combine_datasets(X, y)
>>> fixOut = FixOutliers(treatment='remove', target_split=True)
>>> X_new, y_new = fixOut.fit_transform(Data)
>>> print(X.shape, X_new.shape)
(20640, 8) (16232, 8)
>>> print(y.shape, y_new.shape)
(20640,) (16232, 1)
```