# **Plotting Outliers:**  `plot_outliers` Function

## `plot_outliers`: This Function Is Used To Plot Graphs With Outliers Differentiated From Rest Of The Datapoints<br>

<br>

### **Approaches To Mark Outliers:**
1. **Z-Score Method:** $(x - \bar{x})/\sigma$
2. **Inter-Quartile Range:** $x>Q_3, Q_1>x$
3. **Clustering Algorithm - DBScan:** $D(x, y) = \sqrt{\sum_{i=1}^{n}(x_i - y_i)^2}$
4. **Isolation Forest Method:** $S(x, p, h) = 2^{-\frac{E(h(p))}{c(p)}}$

<br>

**Parameters of `plot_outliers`**:
1. __dataset__ (list, tuple, dictionary, numpy.ndarray, pandas.DataFrame) <br>
    - Dataset Given By User

2. __Col_Names__ (str, int, list, tuple) <br>
Default Value: ['every'] <br>
    - Single Columns Can Be Passed As str Or int (If Dataset Is Passed As `pandas.DataFrame` Or `dict` Then Only Value Passed As `str` Will Be Accepted)
    - Multiple Columns Can Be Passed As list or tuple
    - ['every']: If User Passes Default Value That Is ['every'], Then Function Considers All Columns In Dataset

3. __fontsize__ (int) <br>
Default Value: 12 <br>
    - Fontsize Of Circle Marking Outlier Datapoints

4. __z_threshold__ (float) <br>
Default Value: 3 <br>
    - User Specifies Threshold For Z-Score Computation

5. __db_eps__ (float) <br>
Default Value: 0.5
    - The maximum distance between two samples for DBSCAN clustering

6. __db_min_samples__ (int) <br>
Default Value: 2
    - The number of samples in a neighborhood for a point to be considered as a core point in DBSCAN

7. __if_contamination__ (float) <br>
Default Value: 0.01
    - The estimated proportion of outliers in the data for the Isolation Forest method

8. __color__ (str, list) <br>
- Default Value: ['red', 'blue', 'green', 'cyan']
    - User Can Only Pass `List` Of `Length = 4`, Where Zeroth Index Is Used To Mark Outliers Detected By _Z-Score_, First Index Is Used To Mark Outliers Detected By _Inter-Quartile Range_, Second Index Is Used To Mark Outliers Detected By _DBScan_ And Third Index Is Used To Mark Outliers Detected By _Isolation Forest_
    - If User Passes Value As `String`, Then All Outliers Will Be Marked With That Same Color.

9. __index_present__ (bool) <br>
Default Value: False
Possible Values: True and False
    - True: User Specifies That The Dataset Passed Has First Column As Index Column.
    - False: User Specifies That The Dataset Passed Has First Column As An Important Column, Not Index Column.

<br>

**Returned Output by `plot_outliers`**:
- None, It Just Plots The Required Graph.

<br>

**Examples**:

```python
>>> from analytixhero import plot_outliers
>>> from sklearn.datasets import fetch_california_housing
>>> # California Housing Dataset Is An Inbuilt Dataset In Scikit-Learn
>>> X = Cal.data
>>> plot_outliers(X, Col_Names=[0,1])
```
![](https://i.pinimg.com/564x/39/5a/9b/395a9b9287769bb3750c071ab43d87c3.jpg)

<br>

```python
>>> from analytixhero import plot_outliers
>>> from sklearn.datasets import load_iris
>>> # Iris Dataset Is An Inbuilt Dataset In Scikit-Learn
>>> Iris = load_iris()
>>> X = Iris.data
>>> plot_outliers(X, Col_Names=2, z_threshold=1, db_eps=0.1, if_contamination='auto', color=['red', 'blue', 'yellow', 'green'])
```
![](https://i.pinimg.com/564x/18/a7/4b/18a74beaa154d67ca23baed2f481e315.jpg)