# **Transformations On Dataset:**  `Transform` Class

## `Transform`: Used To Transform Different Columns Of Dataset <br>
### Columns Of Dataset Can Be Transformed On The Basis Of Many Different Mathematical Functions <br>
<i>These Transformations Can Be Used To Decrease Skewness, Kurtosis, Increase Correlation, Normalizing Data, Stabilizing Variance, Increase Interpretability etc.</i>


<br>

### **Different Functions To Transform Dataset:**
1. **Log Transformation:** $log(x)$
2. **Log1P Transformation:** $log(1+x)$
3. **One Plus Log Transformation:** $1+log(x)$
4. **Box-Cox Transformation:** $y(\lambda) = \begin{cases}
\frac{{x^\lambda - 1}}{{\lambda}}, & \text{if } \lambda \neq 0 \\
\ln(x), & \text{if } \lambda = 0
\end{cases}$
5. **Root Transformation:** $\sqrt[n]{x}$
6. **Exponential Transformation:** $e^x$
7. **Inverse Transformation:** $1/x$
8. **Rank Transformation:** $R(x_i) = \text{{Rank}}(x_i)$
9. **Power Transformation:** $x^n$

<br>

### `Transform` Has A Function Called `fit_transform` 

<br>

**Parameters of `Transform`**:
1. __transform__ (str) <br>
Default Value: 'log' <br>
Possible Values: 'log', 'log1p', '1+log', 'boxcox', 'root', 'exp', 'inverse', 'rank' and 'power'
    - 'log' : Log Transformation
    - 'log1p' : Log1P Transformation
    - '1+log' : One Plus Log Transformation
    - 'boxcox' : Box-Cox Transformation
    - 'root' : Root Transformation
    - 'exp' : Exponential Transformation
    - 'inverse' : Inverse Transformation
    - 'rank' : Rank Transformation
    - 'power' : Power Transformation

2. __n_power__ (float) <br>
Default Value: 2 <br>
    - The Float Value For That Will Be Used For 'power' Transformation / 'root' Transformation
    
3. __index_present__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False` 
    - `False` : User Specifies That Dataset Passed By Him/Her Doesn't Have First Column As An Index Column But An Important Column.
    - `True` :  User Specifies That Dataset Passed By Him/Her Has The First Column As An Index Column.

4. __target_split__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False`
    - `False` : Data Will Be Returned As X (Data) And y (Target)
    - `True` : Data Will Be Returned As Whole One Dataset

5. __delete_old__ (bool) <br>
Default Value: False<br>
Possible Values: True and False
    - True: Deletes The Original Columns After Transformation
    - False: Doesn't Deletes The Original Columns After Transformation

6. __filter_warnings__ (bool) <br>
Default Value: False<br>
Possible Values: True and False
    - True: Filter All The Warnings That Pops While Using This Class
    - False: Doesn't Filter All The Warnings That Pops While Using This Class

<br>

`Transform` Has A Function Called `fit_transform` :
 
**Parameters of `Transform.fit_transform`**:
- Dataset: This Dataset Will Undergo Transformation, Any Supported Data Structures (List, Tuple, Dictionary, Numpy.Ndarray, Pandas.DataFrame)
- Col_Names: Only These Columns Will Undergo Transformation <br>
    - Default Value: ['every']
    - User Can Specify Multiple Columns As A 1-D Tuple/List
    - User Can Specify String Or An Integer If He/She Wishes To Pass Only One Column Name
    - If User Passes ['every'], Then Every Column Will Undergo Outlier Treatment
    - Col_Names As `str` Only Applicable If Dataset Is `dict` or `pandas.DataFrame`
- dataframe: If It's True, Returned Dataset Will Be In The Form Of `pandas.DataFrame`, Else It'll Be `numpy.ndarray`
    - Default Value: True
    - Possible Values: True and False

**Returned Output by `Transform.fit_transform`**: 
- `numpy.ndarray` Or `pandas.DataFrame` After Transformations

<br>

**Examples**:

```python
>>> from analytixhero import Transform
>>> X = {
    'A': [10, 12, 15, 18, 20],
    'B': [2, 9, 8, 4, 4]
}
>>> transForm = Transform(transform='1+log')
>>> X_new = transForm.fit_transform(X)
>>> print(X_new)
    A   A_1+log  B   B_1+log
0  10  3.302585  2  1.693147
1  12  3.484907  9  3.197225
2  15  3.708050  8  3.079442
3  18  3.890372  4  2.386294
4  20  3.995732  4  2.386294
```

<br>

```python
>>> from analytixhero import Transform
>>> X = [[1,2,3,4,5], [10,20,30,40,50]]
>>> transForm = Transform(transform='power', n_power=3, delete_old=True)
>>> X_new = transForm.fit_transform(X, Col_Names=[0,1,2], dataframe=False)
>>> print(X_new)
[[    1     8    27     4     5]
 [ 1000  8000 27000    40    50]]
```