# **Null Values Handling:**  `TreatNull` Class

## `TreatNull`: Used To Treat Null Spaces <br>
### Datapoints Containing Null Spaces Either Can Be Imputed Or Removed

<br>

### `TreatNull` Has A Function Called `fit_transform` 

<br>

**Parameters of `TreatNull`**:
1. __treatment__ (str) <br>
Default Value: 'impute' <br>
Possible Values: 'impute' and 'remove'
    - 'impute' : Null Values Will Be Imputed 
    - 'remove' : Datapoints With Null Values Will Get Removed

2. __imputation__ (str) <br>
Default Value: 'mean' <br>
Possible Values: 'mean', 'median', 'mode' and 'std' 
    - 'mean' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `mean` Value Of The Column Where Null Values Exist.
    - 'median' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `median` Value Of The Column Where Null Values Exist.
    - 'mode' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `mode` Value Of The Column Where Null Values Exist.
    - 'std' : If `treatment` Is Set To 'impute', Then It'll Be Imputed With `Standard Deviation` Value Of The Column Where Null Values Exist.

3. __index_present__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False` 
    - `False` : User Specifies That Dataset Passed By Him/Her Doesn't Have First Column As An Index Column But An Important Column.
    - `True` :  User Specifies That Dataset Passed By Him/Her Has The First Column As An Index Column.

4. __target_split__ (bool) <br>
Default Value: `False` <br>
Possible Values: `True` and `False`
    - `False` : Data Will Be Returned As X (Data) And y (Target)
    - `True` : Data Will Be Returned As Whole One Dataset

<br>

`TreatNull` Has A Function Called `fit_transform` :
 
**Parameters of `TreatNull.fit_transform`**:
- Dataset: This Dataset Will Undergo Null Values Treatment, Supported Data Structures (List, Tuple, Dictionary, Numpy.Ndarray, Pandas.DataFrame)

**Returned Output by `TreatNull.fit_transform`**: 
- `numpy.ndarray` Where Null Values Are Handled

<br>

**Examples**:

```python
>>> from analytixhero import TreatNull
>>> Data = [
    [1,2,3,4,5,6], 
    [1, None, 2, 3, 4, 4],
    [1,2,3,5,6,3],
    [" ", 1,2,3,4,5],
    [1,2,3,4,5,""]
    ]
>>> # TreatNull Also Rectifies Empty Spaces
>>> # Dataset Can Be Passed As A List, Tuple, Dictionary, NumPy.NdArray Or Pandas.DataFrame
>>> treNull = TreatNull(treatment='remove', target_split=True)
>>> # `target_split = True` Will Give 2 Datasets (X, y) 
>>> X, y = treNull.fit_transform(Data)
>>> print(X)
[1. 2. 3. 4. 5. 6.]
>>> print(y)
[1. 2. 3. 5. 6. 3.]
```

<br>

```python
>>> from analytixhero import TreatNull
>>> Data = {
    "A": [1,2,3,4,5],
    "B": [1,"  ", 100, 7, 3],
    "C": [None,4,102,6,1]
}
>>> treNull = TreatNull(treatment='impute', imputation='std')
>>> # Data Will Be Imputed With Standard Deviation
>>> Data_new = treNull.fit_transform(Data)
>>> print(Data_new)
[[  1.           1.          49.20958579]
 [  2.          48.23121396   4.        ]
 [  3.         100.         102.        ]
 [  4.           7.           6.        ]
 [  5.           3.           1.        ]]
```